# IMAGE1 = 'tests/images/ac.png'
# IMAGE2 = 'tests/images/bc.png'
# IMAGE3 = 'tests/images/bb.jpg'
# HEIGHT = 512
# WIDTH = 512
# INFILE = 'tests/test_input.csv'
# OUTFILE = 'tests/test_output.csv'

image1 = 'tests/images/ac.png'
image2 = 'tests/images/bc.png'
image3 = 'tests/images/bb.jpg'
height = 512
width = 512
infile = 'tests/test_input.csv'
outfile = 'tests/test_output.csv'


"""
Program to calulate the similarity score between set of images

Returns:
    output_file with score and execution time
"""

import os
import timeit
import csv
# from skimage.measure import compare_ssim
from skimage.metrics import structural_similarity as compare_ssim
from PIL import Image
import cv2


def find_image_score(img_a, img_b, height, width):
    """
    Calculates the image similarity score and execution time

    Arguments:
        img_a {[image]} -- [image1 from the input file].
        img_b {[image]} -- [image2 from the input file].

    Returns:
        score           -- Score of the similarity between the images compared.
        execution_time  -- Time in secs it takes to run the score for each set of images.
    """
    try:
        start = timeit.default_timer()
        print(f"height: {height} Width: {width}")
        std_dimensions = (height, width)
        img_a_resized = cv2.resize(cv2.imread(img_a), std_dimensions)
        img_b_resized = cv2.resize(cv2.imread(img_b), std_dimensions)

        ssim, _ = compare_ssim(img_a_resized, img_b_resized, full=True, multichannel=True)
        score = round(1 - ssim, 2)

        stop = timeit.default_timer()
        execution_time = round(stop - start, 2)
        return [score, execution_time]

    except Exception as err:
        return f"find_image_score:{err}"


def write_output(infile, outfile, height, width):
    """Writes the output file 'output.csv'
    """
    with open(outfile, 'w') as out_file:
        with open(infile, 'r') as in_file:
            csv_reader = csv.reader(in_file, delimiter=',')
            next(csv_reader)
            out_file_headers = "image1,image2,similar,elapsed"
            out_file.write(out_file_headers + '\n')

            for row in csv_reader:
                image_score = find_image_score(row[0], row[1], height, width)
                print("Score: ", image_score)
                out_file.write(",".join(map(str, row)))
                out_file.write(',' + ','.join(map(str, image_score)) + '\n')
                print(f"Comparison of {row[0]} and {row[1]} is completed\n")
    if os.path.exists(outfile):
        return outfile


def check_file(infile):
    """Checks if the file is empty
    """
    with open(infile, 'r') as input_file:
        if input_file.readline() == '':
            return f"{infile} is empty"


def check_images_in_file(infile):
    """
    Checks the input file

    checks the file contents are actual images,
    checks the images are present in the path

    Returns:
        None -- [returns None if the checks succeeds else returns the error]
    """
    with open(infile, 'r') as input_file:
        reader = csv.reader(input_file, delimiter=',')
        next(reader)
        for row in reader:
            try:
                Image.open(row[0])
                Image.open(row[1])
            except Exception as err:
                return f"check_image: {err}"


def main(infile, outfile, height, width):
    """
    Main function

    checks if the input file exists and if
    check_image is successful, calls the write_output
    """
    if os.path.exists(infile):
        if check_file(infile) and check_images_in_file(infile) is None:
            write_output(infile, outfile, height, width)
    else:
        return f"Input file: {infile} is missing"



# from skimage.measure import compare_ssim
from skimage.metrics import structural_similarity as compare_ssim
from PIL import Image
import cv2
from score.findscore import find_image_score, check_file, check_images_in_file, main, write_output
from tests.test_config import *


def cal_score(img_a, img_b):
    ssim, _ = compare_ssim(img_a, img_b, full=True, multichannel=True)
    score = round(1 - ssim, 2)
    return score


def test_find_image_score_1():
    img_a = cv2.resize(cv2.imread(image1), (height, width))
    img_b = cv2.resize(cv2.imread(image2), (height, width))
    score = cal_score(img_a, img_b)
    assert score == 0.0


def test_find_image_score_2():
    img_a = cv2.resize(cv2.imread(image1), (height, width))
    img_b = cv2.resize(cv2.imread(image3), (height, width))
    score = cal_score(img_a, img_b)
    assert score == 0.66


def test_check_image_1():
    img = Image.open(image1)
    assert img.size == (550, 541)


def test_check_file():
    assert check_file(infile) is None


def test_find_image_score():
    score1, _ = find_image_score(image1, image2, height, width)
    score2, _ = find_image_score(image1, image3, height, width)
    assert score1 == [0.0]
    assert score2 == [0.66]


def test_check_images_in_file():
    assert check_images_in_file(infile) is None


def test_main():
    assert main(infile, outfile, height, width) is None


def test_write_output():
    assert write_output(infile, outfile, height, width) == outfile



"""
    Cli wrapper for findscore
"""
import click
from score.findscore import main as score
from score.configs import config as cfg

def helper():
    """Prints help message
    """
    print("""
        e.g imagescore --infile 'input.csv' --outfile 'output.csv'\n
        The options are:
        --infile | -i [str]    -- [Input file path]
        --outfile | -o [str]   -- [Output file path]
        --height | -h [int]    -- [Optional: height to be resized, default = 4096]
        --width | -w [int]     -- [Optional: width to be resized, default = 4096]
    """)

@click.command()
@click.option('--infile', '-i', help="Enter the input file path")
@click.option('--outfile', '-o', help="Enter the output file path")
@click.option('--height', '-h', type=int, help="[optional] height of the image to be resized, default = 4096")
@click.option('--width', '-w', type=int, help="[optional] Width of the image to be resized, default = 4096")
def main(infile, outfile, height, width):
    """
    This tool compares a set of images and returns the similarity score\n
    e.g imagescore --infile 'input.csv' --outfile 'output.csv'\n
    e.g imagescore -i 'input.csv' -o 'output.csv -h 1024 -w 1024'
    """
    try:
        if height is None and width is None:
            score(infile, outfile, height=cfg.HEIGHT, width=cfg.WIDTH)
        else:
            score(infile, outfile, height, width)
    except Exception:
        helper()
        print("Error: Missing options")



from setuptools import setup, find_packages

requirements = [
    "opencv-python",
    "scikit-image",
    "click"
]
version = '1.1.0'
download_url = f'https://github.com/sandjaie/image_score/archive/{version}.tar.gz'

with open('README.md', 'r', errors='ignore') as fd:
    long_desc = fd.read()

setup(
    name='imagescore',
    description='Calculates the similarity score between images',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    author='sandjaie',
    url='https://github.com/sandjaie/image_score',
    author_email='sandjaie@gmail.com',
    version=version,
    packages=find_packages(),
    install_requires=requirements,
    download_url=download_url,
    license='MIT',
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Environment :: Console',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7'
    ],
    entry_points={
        'console_scripts': [
            'imagescore=score.cli:main'
        ]
    }
)